{% load i18n horizon %}
<div class="well">
<p>
    {% blocktrans %}This Cluster will be started with:{% endblocktrans %}
    <br >
    <b>{% blocktrans %}Plugin{% endblocktrans %}</b>: {{ plugin_name }}
    <br />
    <b>{% blocktrans %}Version{% endblocktrans %}</b>: {{ hadoop_version }}
    <br />
</p>
<p>
    {% blocktrans %}Cluster can be launched using existing Cluster Templates.{% endblocktrans %}
</p>
<p>
    {% blocktrans %}The Cluster object should specify OpenStack Image to boot instances for Cluster.{% endblocktrans %}
</p>
<p>
    {% blocktrans %}User has to choose a keypair to have access to clusters instances.{% endblocktrans %}
</p>
</div>
