{% load i18n sizeformat %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ data_source.name }}</dd>
    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ data_source.tenant_id }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ data_source.id }}</dd>
    <dt>{% trans "Type" %}</dt>
    <dd>{{ data_source.type }}</dd>
    <dt>{% trans "URL" %}</dt>
    <dd>{{ data_source.url }}</dd>
    <dt>{% trans "Description" %}</dt>
    <dd>{{ data_source.description|default:_("None") }}</dd>
    <dt>{% trans "Public" %}</dt>
    <dd>{{ data_source.is_public|yesno }}</dd>
    <dt>{% trans "Protected" %}</dt>
    <dd>{{ data_source.is_protected|yesno }}</dd>
    <dt>{% trans "Create time" %}</dt>
    <dd>{{ data_source.created_at|parse_isotime }}</dd>
  </dl>
</div>
